@@ -15,20 +15,21 @@
 import copy
 import difflib
 import io
-import mock
 import os
 import socket
 
+import mock
+import netaddr
 from neutron.agent.l3 import dvr_edge_router
 from neutron.agent.l3 import dvr_snat_ns
 from neutron.agent.l3 import legacy_router
 from neutron.agent.linux import iptables_manager
-from neutron.plugins.common import constants
+from neutron.conf.agent.l3 import config as l3_config
+from neutron_lib import constants
 from oslo_config import cfg
 from oslo_utils import uuidutils
 
 from neutron_vpnaas.extensions import vpnaas
-from neutron_vpnaas.services.vpn.device_drivers import fedora_strongswan_ipsec
 from neutron_vpnaas.services.vpn.device_drivers import ipsec as openswan_ipsec
 from neutron_vpnaas.services.vpn.device_drivers import libreswan_ipsec
 from neutron_vpnaas.services.vpn.device_drivers import strongswan_ipsec
@@ -39,6 +40,7 @@ FAKE_HOST = 'fake_host'
 FAKE_ROUTER_ID = _uuid()
 FAKE_IPSEC_SITE_CONNECTION1_ID = _uuid()
 FAKE_IPSEC_SITE_CONNECTION2_ID = _uuid()
+FAKE_VPNSERVICE_ID = _uuid()
 FAKE_IKE_POLICY = {
     'ike_version': 'v1',
     'encryption_algorithm': 'aes-128',
@@ -57,7 +59,7 @@ FAKE_IPSEC_POLICY = {
 }
 
 FAKE_VPN_SERVICE = {
-    'id': _uuid(),
+    'id': FAKE_VPNSERVICE_ID,
     'router_id': FAKE_ROUTER_ID,
     'name': 'myvpn',
     'admin_state_up': True,
@@ -137,7 +139,7 @@ OPENSWAN_CONNECTION_DETAILS = '''# rightsubnet=networkA/netmaskA, networkB/netma
     # IPsecPolicys params
     ##########################
     # [transform_protocol]
-    auth=%(auth_mode)s
+    phase2=%(auth_mode)s
     # [encapsulation_mode]
     type=%(encapsulation_mode)s
     # [lifetime_value]
@@ -158,11 +160,11 @@ IPV6_NEXT_HOP = '''# To recognize the given IP addresses in this config
     # rightnexthop is not mandatory for ipsec, so no need in ipv6.'''
 
 EXPECTED_OPENSWAN_CONF = """
-# Configuration for myvpn
+# Configuration for %(vpnservice_id)s
 config setup
     nat_traversal=yes
+    virtual_private=%(virtual_privates)s
 conn %%default
-    ikelifetime=480m
     keylife=60m
     keyingtries=%%forever
 conn %(conn1_id)s
@@ -217,16 +219,15 @@ STRONGSWAN_AUTH_ESP = 'esp=aes128-sha1-modp1536'
 STRONGSWAN_AUTH_AH = 'ah=sha1-modp1536'
 
 EXPECTED_IPSEC_OPENSWAN_SECRET_CONF = '''
-# Configuration for myvpn
-60.0.0.4 60.0.0.5 : PSK "password"
-60.0.0.4 60.0.0.6 : PSK "password"'''
+# Configuration for %s
+60.0.0.4 60.0.0.5 : PSK 0scGFzc3dvcmQ=
+60.0.0.4 60.0.0.6 : PSK 0scGFzc3dvcmQ=''' % FAKE_VPNSERVICE_ID
 
 EXPECTED_IPSEC_STRONGSWAN_CONF = '''
-# Configuration for myvpn
+# Configuration for %(vpnservice_id)s
 config setup
 
 conn %%default
-        ikelifetime=60m
         keylife=20m
         rekeymargin=3m
         keyingtries=1
@@ -284,11 +285,11 @@ include strongswan.d/*.conf
 '''
 
 EXPECTED_IPSEC_STRONGSWAN_SECRET_CONF = '''
-# Configuration for myvpn
-60.0.0.4 60.0.0.5 : PSK "password"
+# Configuration for %s
+60.0.0.4 60.0.0.5 : PSK 0scGFzc3dvcmQ=
 
-60.0.0.4 60.0.0.6 : PSK "password"
-'''
+60.0.0.4 60.0.0.6 : PSK 0scGFzc3dvcmQ=
+''' % FAKE_VPNSERVICE_ID
 
 PLUTO_ACTIVE_STATUS = """000 "%(conn_id)s/0x1": erouted;\n
 000 #4: "%(conn_id)s/0x1":500 STATE_QUICK_R2 (IPsec SA established);""" % {
@@ -319,13 +320,14 @@ class BaseIPsecDeviceDriver(base.BaseTestCase):
               vpnservice=FAKE_VPN_SERVICE):
         super(BaseIPsecDeviceDriver, self).setUp()
         for klass in [
-            'neutron.common.rpc.create_connection',
+            'neutron.common.rpc.Connection',
             'oslo_service.loopingcall.FixedIntervalLoopingCall'
         ]:
             mock.patch(klass).start()
         self._execute = mock.patch.object(ipsec_process, '_execute').start()
         self.agent = mock.Mock()
         self.conf = cfg.CONF
+        l3_config.register_l3_agent_config_opts(l3_config.OPTS, self.conf)
         self.agent.conf = self.conf
         self.driver = driver(
             self.agent,
@@ -343,8 +345,7 @@ class BaseIPsecDeviceDriver(base.BaseTestCase):
     @staticmethod
     def generate_diff(a, b):
         """Generates unified diff of a and b."""
-        by_lines = lambda x: x.splitlines(True)
-        a, b = list(by_lines(a)), list(by_lines(b))
+        a, b = list(a.splitlines(True)), list(b.splitlines(True))
         diff = difflib.unified_diff(a, b, fromfile="expected",
                                     tofile="actual")
         return diff
@@ -422,7 +423,8 @@ class IPSecDeviceLegacy(BaseIPsecDeviceDriver):
         self._make_router_info_for_test()
 
     def _make_router_info_for_test(self):
-        self.router = legacy_router.LegacyRouter(FAKE_ROUTER_ID,
+        self.router = legacy_router.LegacyRouter(router_id=FAKE_ROUTER_ID,
+                                                 agent=self.agent,
                                                  **self.ri_kwargs)
         self.router.router['distributed'] = False
         self.router.iptables_manager.ipv4['nat'] = self.iptables
@@ -959,6 +961,7 @@ class TestOpenSwanConfigGeneration(BaseIPsecDeviceDriver):
                 'ike_lifetime': 3600,
                 'life_time': 3600,
                 'encapsulation_mode': 'tunnel'}
+        virtual_privates = []
         # Convert local CIDRs into assignment strings. IF more than one,
         # pluralize the attribute name and enclose in brackets.
         cidrs = info.get('local_cidrs', [['10.0.0.0/24'], ['11.0.0.0/24']])
@@ -968,15 +971,25 @@ class TestOpenSwanConfigGeneration(BaseIPsecDeviceDriver):
                 local_cidrs.append("s={ %s }" % ' '.join(cidr))
             else:
                 local_cidrs.append("=%s" % cidr[0])
+            for net in cidr:
+                version = netaddr.IPNetwork(net).version
+                virtual_privates.append('%%v%s:%s' % (version, net))
         # Convert peer CIDRs into space separated strings
         cidrs = info.get('peer_cidrs', [['20.0.0.0/24', '30.0.0.0/24'],
                                         ['40.0.0.0/24', '50.0.0.0/24']])
+        for cidr in cidrs:
+            for net in cidr:
+                version = netaddr.IPNetwork(net).version
+                virtual_privates.append('%%v%s:%s' % (version, net))
         peer_cidrs = [' '.join(cidr) for cidr in cidrs]
         local_ip = info.get('local', '60.0.0.4')
         version = info.get('local_ip_vers', 4)
         next_hop = IPV4_NEXT_HOP if version == 4 else IPV6_NEXT_HOP % local_ip
         peer_ips = info.get('peers', ['60.0.0.5', '60.0.0.6'])
+        virtual_privates.sort()
         return EXPECTED_OPENSWAN_CONF % {
+            'vpnservice_id': FAKE_VPNSERVICE_ID,
+            'virtual_privates': ','.join(virtual_privates),
             'next_hop': next_hop,
             'local_cidrs1': local_cidrs[0], 'local_cidrs2': local_cidrs[1],
             'local_ver': version,
@@ -1057,6 +1070,7 @@ class IPsecStrongswanConfigGeneration(BaseIPsecDeviceDriver):
         peer_ips = info.get('peers', ['60.0.0.5', '60.0.0.6'])
         auth_mode = info.get('ipsec_auth', STRONGSWAN_AUTH_ESP)
         return EXPECTED_IPSEC_STRONGSWAN_CONF % {
+            'vpnservice_id': FAKE_VPNSERVICE_ID,
             'local_cidrs1': local_cidrs[0], 'local_cidrs2': local_cidrs[1],
             'peer_cidrs1': peer_cidrs[0], 'peer_cidrs2': peer_cidrs[1],
             'left': local_ip,
@@ -1407,61 +1421,96 @@ class TestLibreSwanProcess(base.BaseTestCase):
                                                        self.vpnservice,
                                                        mock.ANY)
 
-    @mock.patch('os.remove')
     @mock.patch('os.path.exists', return_value=True)
-    def test_ensure_configs_on_restart(self, exists_mock, remove_mock):
+    def test_ensure_configs_on_restart(self, exists_mock):
         openswan_ipsec.OpenSwanProcess.ensure_configs = mock.Mock()
-        with mock.patch.object(self.ipsec_process, '_execute') as fake_execute:
+        with mock.patch.object(
+            self.ipsec_process, '_execute'
+        ) as fake_execute, mock.patch.object(
+            self.ipsec_process, '_ipsec_execute'
+        ) as fake_ipsec_execute, mock.patch.object(
+            self.ipsec_process, '_ensure_needed_files'
+        ) as fake_ensure_needed_files:
             self.ipsec_process.ensure_configs()
-            expected = [mock.call(['chown', '--from=%s' % os.getuid(),
-                                   'root:root',
-                                   self.ipsec_process._get_config_filename(
-                                       'ipsec.secrets')]),
-                        mock.call(['ipsec', '_stackmanager', 'start']),
-                        mock.call(['ipsec', 'checknss',
-                                   self.ipsec_process.etc_dir])]
-            fake_execute.assert_has_calls(expected)
-            self.assertEqual(3, fake_execute.call_count)
-            self.assertTrue(exists_mock.called)
-            self.assertTrue(remove_mock.called)
 
-    @mock.patch('os.remove')
+            expected = [mock.call(['rm', '-f',
+                                   self.ipsec_process._get_config_filename(
+                                       'ipsec.secrets')]),
+                        mock.call(['chown', '--from=%s' % os.getuid(),
+                                   'root:root',
+                                   self.ipsec_process._get_config_filename(
+                                       'ipsec.secrets')]),
+                        mock.call(['chown', '--from=%s' % os.getuid(),
+                                   'root:root', self.ipsec_process.log_dir])]
+            fake_execute.assert_has_calls(expected)
+            self.assertEqual(3, fake_execute.call_count)
+
+            expected = [mock.call(['_stackmanager', 'start']),
+                        mock.call(['checknss'])]
+            fake_ipsec_execute.assert_has_calls(expected)
+            self.assertEqual(2, fake_ipsec_execute.call_count)
+
+            self.assertTrue(fake_ensure_needed_files.called)
+            self.assertTrue(exists_mock.called)
+
     @mock.patch('os.path.exists', return_value=False)
-    def test_ensure_configs(self, exists_mock, remove_mock):
+    def test_ensure_configs(self, exists_mock):
         openswan_ipsec.OpenSwanProcess.ensure_configs = mock.Mock()
-        with mock.patch.object(self.ipsec_process, '_execute') as fake_execute:
+        with mock.patch.object(
+            self.ipsec_process, '_execute'
+        ) as fake_execute, mock.patch.object(
+            self.ipsec_process, '_ipsec_execute'
+        ) as fake_ipsec_execute, mock.patch.object(
+            self.ipsec_process, '_ensure_needed_files'
+        ) as fake_ensure_needed_files:
             self.ipsec_process.ensure_configs()
+
             expected = [mock.call(['chown', '--from=%s' % os.getuid(),
                                    'root:root',
                                    self.ipsec_process._get_config_filename(
                                        'ipsec.secrets')]),
-                        mock.call(['ipsec', '_stackmanager', 'start']),
-                        mock.call(['ipsec', 'checknss',
-                                   self.ipsec_process.etc_dir])]
+                        mock.call(['chown', '--from=%s' % os.getuid(),
+                                   'root:root', self.ipsec_process.log_dir])]
             fake_execute.assert_has_calls(expected)
-            self.assertEqual(3, fake_execute.call_count)
+            self.assertEqual(2, fake_execute.call_count)
+
+            expected = [mock.call(['_stackmanager', 'start']),
+                        mock.call(['checknss'])]
+            fake_ipsec_execute.assert_has_calls(expected)
+            self.assertEqual(2, fake_ipsec_execute.call_count)
+
+            self.assertTrue(fake_ensure_needed_files.called)
             self.assertTrue(exists_mock.called)
-            self.assertFalse(remove_mock.called)
 
         exists_mock.reset_mock()
-        remove_mock.reset_mock()
 
-        with mock.patch.object(self.ipsec_process, '_execute') as fake_execute:
-            fake_execute.side_effect = [None, None, RuntimeError, None]
+        with mock.patch.object(
+            self.ipsec_process, '_execute'
+        ) as fake_execute, mock.patch.object(
+            self.ipsec_process, '_ipsec_execute'
+        ) as fake_ipsec_execute, mock.patch.object(
+            self.ipsec_process, '_ensure_needed_files'
+        ) as fake_ensure_needed_files:
+            fake_ipsec_execute.side_effect = [None, RuntimeError, None]
             self.ipsec_process.ensure_configs()
+
             expected = [mock.call(['chown', '--from=%s' % os.getuid(),
                                    'root:root',
                                    self.ipsec_process._get_config_filename(
                                        'ipsec.secrets')]),
-                        mock.call(['ipsec', '_stackmanager', 'start']),
-                        mock.call(['ipsec', 'checknss',
-                                   self.ipsec_process.etc_dir]),
-                        mock.call(['ipsec', 'initnss',
-                                   self.ipsec_process.etc_dir])]
+                        mock.call(['chown', '--from=%s' % os.getuid(),
+                                   'root:root', self.ipsec_process.log_dir])]
             fake_execute.assert_has_calls(expected)
-            self.assertEqual(4, fake_execute.call_count)
+            self.assertEqual(2, fake_execute.call_count)
+
+            expected = [mock.call(['_stackmanager', 'start']),
+                        mock.call(['checknss']),
+                        mock.call(['initnss'])]
+            self.assertEqual(3, fake_ipsec_execute.call_count)
+            fake_ipsec_execute.assert_has_calls(expected)
+
+            self.assertTrue(fake_ensure_needed_files.called)
             self.assertTrue(exists_mock.called)
-            self.assertFalse(remove_mock.called)
 
 
 class IPsecStrongswanDeviceDriverLegacy(IPSecDeviceLegacy):
@@ -1500,20 +1549,3 @@ class IPsecStrongswanDeviceDriverDVR(IPSecDeviceDVR):
               ipsec_process=strongswan_ipsec.StrongSwanProcess):
         super(IPsecStrongswanDeviceDriverDVR, self).setUp(driver,
                                                           ipsec_process)
-
-
-class IPsecFedoraStrongswanDeviceDriverLegacy(
-    IPsecStrongswanDeviceDriverLegacy):
-
-    def setUp(self, driver=fedora_strongswan_ipsec.FedoraStrongSwanDriver,
-              ipsec_process=fedora_strongswan_ipsec.FedoraStrongSwanProcess):
-        super(IPsecFedoraStrongswanDeviceDriverLegacy,
-              self).setUp(driver, ipsec_process)
-
-
-class IPsecFedoraStrongswanDeviceDriverDVR(IPSecDeviceDVR):
-
-    def setUp(self, driver=fedora_strongswan_ipsec.FedoraStrongSwanDriver,
-              ipsec_process=fedora_strongswan_ipsec.FedoraStrongSwanProcess):
-        super(IPsecFedoraStrongswanDeviceDriverDVR, self).setUp(driver,
-                                                                ipsec_process)
